"""
Zoom API Helper - Utilities to interact with the Zoom API v2
"""

__title__ = 'zoom-api-helper'
__description__ = 'Utilities to interact with the Zoom API v2'
__url__ = 'https://github.com/rnag/zoom-api-helper'
__version__ = '0.1.1'
__author__ = 'Ritvik Nag'
__author_email__ = 'rv.kvetch@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 Ritvik Nag'
